/**
 * Module to manage the WiFly device. Versions 2.21 or higher
 *
 * Juan E. Gómez Pérez
 * UOC - TFG Sistemas Embebidos
 *
 * revision 1.0 date 20/10/2015
 * revision 1.1 data 13/11/2015: included HTTP module in wifly functions. Define atomic functions.
 */

#ifndef WIFLY_LIB_INC_WIFLY_H_
#define WIFLY_LIB_INC_WIFLY_H_


#define WIFLY_PORT	0
#define WIFLY_PIN	22

#define WIFLY_MAXCONNTRIES	3

#define WIFLY_WAIT_SMALL_TIME	400		// Time to wait for receiving an AOK
#define WIFLY_WAIT_BIG_TIME		3000	// Time to wait for a bunch of characters, no determined
#define WIFLY_CMODE "$$$"  				// Wifly Command mode characters

#define MAX_BUFFER 256					// Standard CHAR buffer size

/** Initialize WiFly Module
 *
 * @PortNum UART Port to communicate with WiFly module
 * @baudrate UART speed to communicate with WiFly module
 *
 * @return always returns 1 pending to check for error and return 0 if error
 */
uint32_t Wifly_Init(uint32_t PortNum, uint32_t baudrate);

/** Set WiFly wireless SSID to connect to
 *
 * @ssid string with the desired SSID
 *
 * @return 1 if success with setting SSID, 0 otherwise.
 */
uint8_t Wifly_SetSSID(const char *ssid);

/** Send a command to WiFly and check if it returns AOK
 *
 * @comando Main command to send to WiFly
 * @parametro Parameter of the command to send to WiFly
 *
 * @return 1 if the command returned AOK, or 0 otherwise.
 *
 */
uint8_t Wifly_SendCommandAOK(const char *comando, const char *parametro);

/** WiFly sets WLAN pass phrase
 *
 * @clave string with the wlan pass phrase
 *
 * @return 1 if success with setting passphrase, 0 otherwise.
 */
uint8_t Wifly_SetPassword(const char *clave);

/** Set WiFly DHCP client mode
 *
 * @activar  [0 = disable, 1=enable]
 *
 * @return 1 if the set was successful, or 0 if error
 */
uint8_t Wifly_DHCP(const char *activar);

/** Join WiFly to the defined wifi network
 * Pending to check if the SSID, passphrase and authmode have been defined <<<<****<<<<
 * Pending to check if the return value contains "Associated!" to validate successful association <<<<****<<<<
 *
 * @ssid Wireless network SSID to connect with
 *
 * @return Always return 1
 */

int8_t Wifly_Connect(const char *ssid,const char *clave, const char *tipoAuth);

/** Set WiFly wifi authentication mode
 * Pending to check if the value sent is on the allowed range <<<<****<<<<
 *
 * @clave Auth tipe, 4 for WPA2-PSK
 *
 * @return 1 if the set was successful, or 0 if error
 */
uint8_t Wifly_SetAuth(const char *tipo);

/** WiFly pings a hostname or host IP address
 * Pending to check if ping was sucessful <<<<<******<<<<<
 *
 * @host string with the host name or host IP address
 *
 * @return always return 1
 */
uint8_t Wifly_Ping(const char *host);

/** Gets current WiFly IP address
 *
 * @ip Variable in which we will write the WiFly gotten ip
 *
 * @return void
 */
void Wifly_GetIP(char *ip);

/** Get current WiFly wifi channel in use
 *
 * @No Param
 *
 * @return channel number (integer mode)
 */
uint8_t Wifly_GetChannel(void);

/** Get current WiFly RSSI (it can be negative, even it can be possitive? ;-)
 *
 * @No Param
 *
 * @return RSSI value in integer form (because it usually is negative)
 */
int Wifly_GetRSSI(void);

/** Get current WiFly uptime since last reset
 *
 * @No Parm
 *
 * @return Number or senconds since last reset
 */
int Wifly_GetUptime(void);

/** Check if the Wifly Module is connected
 *
 * No Params
 *
 * @return 		1 if we're associated, -1 if we're not associated
 *
 */

int Wifly_AmIConnected(void);


/** Initialize WiFly HTTP module, configure it for HTTP/TCP requests
 *
 * @HTTPort		HTTP Port in which we will issue the commands
 * @dns_name	Host name to which we will send the commands
 *
 * @return 		1 if all the commands were successful, -1 if error
 */
int8_t Wifly_HTTPInit(uint8_t HTTPPort, char *dns_name);

/** Issue a GET command against a host
 *
 * @host				Name of the host which will open the http connection to
 * @URI					URI of the host which we will request. Be careful if you need host name included, then it's not URI but URL ;-)
 * @httpget_buffer		char buffer to get the returned data.
 *
 * @return 				-1 if error, or 1 if success
 */
int8_t Wifly_HTTPGet(char *host,char *URI,char *httpget_buffer);

#endif /* WIFLY_LIB_INC_WIFLY_H_ */
