// --- gpio.c ---


#include "LPC17xx.h"
#include "gpio.h"
#include "utils.h"

const uint8_t	gPINSEL0[]={0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30};
const uint8_t	gPINSEL1[]={0,2,4,6,8,10,12,14,16,18,20,22,24,26,28};
const uint8_t	gPINSEL2[]={0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30};
const uint8_t	gPINSEL3[]={0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30};

/** Initialize a GPIO pin using the requested configurations
 *
 *
 * @pinSel		Number of PinSel to be used this depends on which port has been choosed
 * @psel_Port	Port to be initizlized on the PINSEL
 * @pinFunction	two bits of information selecting the pin configuration
 * @pinMode		Mode to configure the pin, pull-up, pull-down, etc. Depends on pin check manual
 *
 * @return 		-1 on error requesting pin configuration, and 1 on success
 */
int8_t initialize_PIN(uint8_t pinSel, uint8_t psel_Port, uint8_t pinFunction, uint8_t pinMode);


/************************************************/
/*                Main functions                */
/************************************************/

// Initialize PINSEL
// pinSel [0-3]
// psel_Port depends on pinsel
// pinFunction	[00 / 01 / 10 / 11] it switches among GPIO, ADC, I2c ....
// pin_Mode [00 / 01 / 10 / 11]
int8_t initialize_PIN(uint8_t pinSel, uint8_t psel_Port, uint8_t pinFunction, uint8_t pinMode)
{
	if (pinSel>3) return -1;   // We are just able to initialize from PINSEL0 to PINSEL3
	if (pinFunction>3) return -1; 	// PINFUNCTION cannot be higher than 0x11b
	if (pinMode>3)	return -1; 	// PINMODE cannot be higher than 0x11b

	switch (pinSel)
	{
		case 0:
			if ((psel_Port >=0 && psel_Port<=11) || (psel_Port==15))
			{	// We're ok, let's go
				// Select PIN
				//LPC_PINCON->PINSEL0 |= ( ~( pinFunction << gPINSEL0[psel_Port] ) );
				//LPC_PINCON->PINMODE0 |= (pinMode << gPINSEL0[psel_Port]);
				LPC_PINCON->PINSEL0 &= ( ~ ( 0b11 << gPINSEL0[psel_Port]));  // Put both bits to setup with 0
				LPC_PINCON->PINSEL0 |= (( pinFunction << gPINSEL0[psel_Port] ));  // Set final pinsel config
				LPC_PINCON->PINMODE0 |= (pinMode << gPINSEL0[psel_Port]);
			}	else
					return -1; // He's trying to initialize something that does not exists
			break;
		case 1:
			if (psel_Port >=16 && psel_Port<=30)
			{
				LPC_PINCON->PINSEL1 &= ( ~ ( 0b11 << gPINSEL1[psel_Port-16]));  // Put both bits to setup with 0
				LPC_PINCON->PINSEL1 |= (( pinFunction << gPINSEL1[psel_Port-16] ));
				LPC_PINCON->PINMODE1 |= (pinMode << gPINSEL1[psel_Port-16]);
			}	else
					return -1; // He's trying to initialize something that does not exists
			break;
		case 2:
			if ((psel_Port >=0 && psel_Port<=1) || (psel_Port==4) || (psel_Port>=8 && psel_Port<=10) || (psel_Port>=14 && psel_Port<=15))
			{
				LPC_PINCON->PINSEL2 |= ( ~( pinFunction << gPINSEL2[psel_Port] ) );
				LPC_PINCON->PINMODE2 |= (pinMode << gPINSEL2[psel_Port]);
			}	else
					return -1; // He's trying to initialize something that does not exists
			break;
		case 3:
			if (psel_Port >=16 && psel_Port<=31)
			{
				LPC_PINCON->PINSEL3 |= ( ~( pinFunction << gPINSEL3[psel_Port-16] ) );
				LPC_PINCON->PINMODE3 |= (pinMode << gPINSEL3[psel_Port-16]);
			}	else
					return -1; // He's trying to initialize something that does not exists
			break;
		default:
			return -1;			// We never should reach here
	}
	return 1;
}

void ADC_init(uint8_t adc_Port)
{

	uint32_t pclk, pclkdiv, clkdiv;

	// Switch on the ADC
	LPC_SC->PCONP |= (1<<12);

	// Initialize te pins
	switch (adc_Port)
	{
		case 0:
			initialize_PIN(1,23,0b01,0);
			break;
		case 1:
			initialize_PIN(1,24,0b01,0);
			break;
		case 2:
			initialize_PIN(1,25,0b01,0);
			break;
		case 3:
			initialize_PIN(1,26,0b01,0);
			break;
		case 4:  // AD0.4
			initialize_PIN(3,30,0b11,0);
			break;
		case 5:
			initialize_PIN(3,31,0b11,0);
			break;
		case 6:
			initialize_PIN(0,3,0b10,0);
			break;
		case 7:
			initialize_PIN(0,2,0b10,0);
			break;
	}

	// Let's configure the clock
	pclkdiv = (LPC_SC->PCLKSEL0 >> 22) & 0x03;   // Get the bits 24 and 25 on the right side, and make the rest 0
	switch ( pclkdiv )
	{
		case 0x00:
		default:
			pclk = SystemCoreClock/4;
			break;
		case 0x01:
			pclk = SystemCoreClock;
			break;
		case 0x02:
			pclk = SystemCoreClock/2;
			break;
		case 0x03:
			pclk = SystemCoreClock/8;
			break;
	}

	clkdiv = ((pclk-1) / 13000000)+1; // Cannot be higher thatn 13Mhz.

	// Choose which ADC we're going to use
	LPC_ADC -> ADCR = (1 << 0)
						| (clkdiv << 8) 	// Clock to use
						| (0 << 16) 		// No BURST
						| (1 << 21)			// Enable ADC conversions
						| (0 << 24)			// We use manual Conversion, switch off
						| (0 << 27);

//	LPC_ADC->ADCR &= ~( (7 << 24) | 0xFF );
//	LPC_ADC->ADCR &= ~( 1 << 16 );		// Choose No burst


}

uint16_t ADC_read(uint8_t adc_Port)
{
	uint32_t	valorCanal=0;
	// Start conversion now
	LPC_ADC->ADCR |= (( 1 << 24) | ( 1 << adc_Port));   // Enable conversion and define port

	switch(adc_Port)
	{
		case 0:
			while (((LPC_ADC->ADDR0 & (1 << 31)) == 0));  // Wait for ADC Finish conversion (active waiting), wait for bit 31 to be enabled

			// we shift bits to start reading the 12bits data from bit4 to bit 15
			valorCanal = ((LPC_ADC->ADDR0 >>4) & 0xfff);
			break;
		case 1:
			while (((LPC_ADC->ADDR1 & (1 << 31)) == 0));  // Wait for ADC Finish conversion (active waiting), wait for bit 31 to be enabled

			// we shift bits to start reading the 12bits data from bit4 to bit 15
			valorCanal = ((LPC_ADC->ADDR1 >>4) & 0xfff);
			break;
		case 6:
			while (((LPC_ADC->ADDR6 & (1 << 31)) == 0));  // Wait for ADC Finish conversion (active waiting), wait for bit 31 to be enabled
			// we shift bits to start reading the 12bits data from bit4 to bit 15
			valorCanal = ((LPC_ADC->ADDR6 >>4) & 0xfff);
			break;

	}

	// Disable conversion
	 LPC_ADC->ADCR &= ~( (7 << 24) | 0xFF );

	return valorCanal;
}


void GPIO_pin_on(uint8_t gpio_Port, uint8_t gpio_Pin)
{

	uint8_t	gPsel = 0;
	if ((gpio_Port==0) && (gpio_Pin>=0 && gpio_Pin<=15))	gPsel=0;
	if ((gpio_Port==0) && (gpio_Pin>=16 && gpio_Pin<=30))	gPsel=1;
	if ((gpio_Port==1) && (gpio_Pin>=0 && gpio_Pin<=15))	gPsel=2;
	if ((gpio_Port==1) && (gpio_Pin>=16 && gpio_Pin<=30))	gPsel=3;

	// Initialize port
	initialize_PIN(gPsel,gpio_Pin,0b00,0b00);
	switch(gpio_Port)
	{
		case 0:
			LPC_GPIO0->FIODIR |= (1 << gpio_Pin); 		// Set this pin as output (1)
			LPC_GPIO0->FIOSET  = (1 << gpio_Pin);		// Raise signal on this pin
			break;
		// pending the other cases
	}

}

void GPIO_pin_off(uint8_t gpio_Port, uint8_t gpio_Pin)
{
	uint8_t	gPsel = 0;
	if ((gpio_Port==0) && (gpio_Pin>=0 && gpio_Pin<=15))	gPsel=0;
	if ((gpio_Port==0) && (gpio_Pin>=16 && gpio_Pin<=30))	gPsel=1;
	if ((gpio_Port==1) && (gpio_Pin>=0 && gpio_Pin<=15))	gPsel=2;
	if ((gpio_Port==1) && (gpio_Pin>=16 && gpio_Pin<=30))	gPsel=3;

	// Initialize port
	initialize_PIN(gPsel,gpio_Pin,0b00,0b00);
	switch(gpio_Port)
	{
	case 0:
		LPC_GPIO0->FIODIR |= (1 << gpio_Pin); 		// Set this pin as output (1)
		LPC_GPIO0->FIOCLR  = (1 << gpio_Pin);		// Lower signal on this pin
		break;
	}

}

// This function lowers the GPIO port during 1ms and raises again
void	GPIO_send_reset_signal(uint8_t gpio_Port, uint8_t gpio_Pin)
{
	uint8_t	gPsel = 0;
	if ((gpio_Port==0) && (gpio_Pin>=0 && gpio_Pin<=15))	gPsel=0;
	if ((gpio_Port==0) && (gpio_Pin>=16 && gpio_Pin<=30))	gPsel=1;
	if ((gpio_Port==1) && (gpio_Pin>=0 && gpio_Pin<=15))	gPsel=2;
	if ((gpio_Port==1) && (gpio_Pin>=16 && gpio_Pin<=30))	gPsel=3;

	// Initialize port
	initialize_PIN(gPsel,gpio_Pin,0b00,0b00);
	switch(gpio_Port)
	{
	case 0:
		LPC_GPIO0->FIODIR |= (1 << gpio_Pin); 		// Set this pin as output (1)
		LPC_GPIO0->FIOCLR  = (1 << gpio_Pin);		// Lower signal on this pin
			Utils_DelayMs(1);
		LPC_GPIO0->FIOSET  = (1 << gpio_Pin);		// Raise signal on this pin
		break;
	}
}
