// --- wifly.c ---

#include "LPC17xx.h"
#include "FreeRTOS.h"

#include <task.h>
#include <semphr.h>

#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include "uart.h"
#include "gpio.h"
#include "printf.h"
#include "log.h"
#include "wifly.h"
#include "utils.h"

#define wDEBUG 1
#define LOGSERIALPORT	3
#define LOGSERIALSPEED 9600

#define DHCP "1"

/* Cosas que necesito controlar */
static int8_t WiFly_SerialPort = -1;   // -1 because 0 can be UART 0, fixed  bug
static uint8_t incommandmode = 0;

static char buffer_RX[MAX_BUFFER];
static char buffer[MAX_BUFFER];

//int8_t		wifly_initialized = 0 ;
int8_t		http_Initialized = -1;
static char	http_RX[1024];

static xSemaphoreHandle xMutexWiFly;

/* Define local Wifly functions */

uint8_t Wifly_CommandMode(void);
void Wifly_SetNoCommmandMode(void);
uint8_t Wifly_ExitCommandMode(void);
uint8_t Wifly_SendCommand(const char *comando, const char *parametro, uint32_t demora, char *retorno);
void Wifly_ClearBuffers(void);


uint32_t Wifly_Init(uint32_t PortNum, uint32_t baudrate)
{

	if (WiFly_SerialPort == -1 ) WiFly_SerialPort = PortNum;

	// Initialize Mutex
	xMutexWiFly = xSemaphoreCreateMutex();

	// Get Mutex to initialize Wifly
	xSemaphoreTake( xMutexWiFly, portMAX_DELAY );

	/* Initialize Serial COMM to communicate with WiFly */
	Printf_Start(WiFly_SerialPort,baudrate);

	/* This should be in a different module in library */
	GPIO_send_reset_signal(WIFLY_PORT,WIFLY_PIN);
	Utils_DelayMs(1500);

	/* Pendiente gestionar semaforos y ver si está inicializado */
	//Wifly_ClearBuffers();

	if (wDEBUG) Log_Start(LOGSERIALPORT,LOGSERIALSPEED);

	Printf_ScanCompleto(WiFly_SerialPort,3000,buffer_RX); /* Get returned command */
	if (wDEBUG) Log_Log("*** Wifly Module Reset ***\r\n %s",buffer_RX);

	/* Check if we have to debug, and initialize Serial Port to Debug */
	if (wDEBUG)	Log_Log("WiFLY RX: Initializing port %i with %i bps\r\n",PortNum, baudrate);

	xSemaphoreGive( xMutexWiFly );

	/* We're in command mode! */
//	if (wifly_initialized == 0)
//	{
		Wifly_CommandMode();
		/* Set UART to no echo, we do not want to read what we write */
		Printf_Print(WiFly_SerialPort,"set uart mode  1\r");
		Printf_Print(WiFly_SerialPort,"\r");
		Printf_ScanCompleto(WiFly_SerialPort,500,buffer_RX);
		if(strcmp(buffer_RX,"<4.41>") == 0) if (wDEBUG) Log_Log("COMMAND Mode successful exit");
//		wifly_initialized = 1;
		Wifly_ExitCommandMode();
//	}

	return 1;
}

/*
 * Join to a WiFY Network
 */
int8_t Wifly_Connect(const char *ssid,const char *clave, const char *tipoAuth)
{

	char *strOK;
	uint8_t tries=0;
	int8_t conn = 0;

	while ((tries < WIFLY_MAXCONNTRIES) && ((conn == -1) || (conn == 0)))
	{
		// Enter command mode (and get mutex)
		if(Wifly_CommandMode() == 1)
		{
			if(Wifly_SendCommandAOK("set ip dhcp ",DHCP)!=1) { conn = -1; } // Error
			if (conn == 0) if(Wifly_SendCommandAOK("set wlan ssid ",ssid) != 1) { conn = -1; } // Error
			if (conn == 0) if(Wifly_SendCommandAOK("set wlan passphrase ",clave)!=1) { conn = -1; } // Error
			if (conn == 0) if(Wifly_SendCommandAOK("set wlan auth ",tipoAuth)!=1) { conn = -1; } // Error
			if (conn == 0) if(Wifly_SendCommand("join ",ssid,WIFLY_WAIT_BIG_TIME*4,buffer)!=1) { conn = -1; } // Error
		} else conn = -1;

		// Wait a bit for join negotiation
		//Utils_DelayMs(10000);
			// We should check the Join command return
		if (wDEBUG) Log_Log("WiFLY RX: (join)\r\n%s",buffer);
		strOK = strstr(buffer,"Associated!");
		if ((strOK == NULL) || (conn == -1)) // We're not joined or we got an error
		{
			if(wDEBUG) Log_Log("WiFLY INF: Error associating AP");
			// Retry
			GPIO_send_reset_signal(WIFLY_PORT,WIFLY_PIN);
			tries++;
			conn = 0; // Restart
			Wifly_SetNoCommmandMode();
		}
		else
		{	// If we get here success
			if (wDEBUG) Log_Log("WiFLY INF: Associated!");
			conn = 1;
			Wifly_ExitCommandMode();
		}

	}

	if (conn == 0) conn=-1;
	return conn; // OK

}

/**
 *
 * We change the approach, we will atomic operations by getting mutex on commend enter
 * and releasing the mutex when exiting from command mode
 *
 */
uint8_t Wifly_CommandMode(void)
{

	/* Am I in command mode? */
	if (incommandmode == 1) return 1;

	xSemaphoreTake( xMutexWiFly, portMAX_DELAY );
	{

		Wifly_ClearBuffers();
		UARTClear();

		Printf_Print(WiFly_SerialPort,WIFLY_CMODE);
		Printf_ScanCompleto(WiFly_SerialPort,500,buffer_RX);

		if (wDEBUG) Log_Log("WiFLY RX: %s\r\n",buffer_RX);
		if (strcmp(buffer_RX,"CMD\r\n") == 0)
		{
			if (wDEBUG) Log_Log("Wifly We're in command mode.\r\n");
			incommandmode = 1;
		}

		// We will release the semaphore in exit command mode
		// xSemaphoreGive(xMutexWiFly);
	}
	return incommandmode;
}

uint8_t Wifly_ExitCommandMode(void)
{
	Wifly_ClearBuffers();
	UARTClear();

	/* Am I in command mode? */
	if (incommandmode != 1) return 1;

	Printf_Print(WiFly_SerialPort,"exit\r");
	Printf_ScanCompleto(WiFly_SerialPort,1000,buffer);
	if (wDEBUG) Log_Log("WiFLY RX: %s\r\n",buffer);
	if (strcmp(buffer,"EXIT\r\n") == 0)
	{
		/* We're not in command mode! */
		incommandmode = 0;
	}

	// We release command mode mutex
	xSemaphoreGive(xMutexWiFly);

	return incommandmode;
}

void Wifly_SetNoCommmandMode(void)
{
	incommandmode =0;
	xSemaphoreGive(xMutexWiFly);

}

uint8_t Wifly_Ping(const char *host)
{
	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); return -1; } // Error
		Wifly_SendCommand("ping ",host, WIFLY_WAIT_BIG_TIME, buffer_RX);
	Wifly_ExitCommandMode();
	return 1;
}

void Wifly_GetIP(char *ip)
{

	char* cadenaBuscada="IP";
	char* cadenaSplit=":";

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); sprintf(ip,"Error"); } // Error
	else
	{
		Wifly_SendCommand("get ip",NULL,WIFLY_WAIT_SMALL_TIME, buffer_RX);
		// ipL = buscar_cadena(buffer_RX,cadenaBuscada);
		char* salida = strstr(buffer_RX,cadenaBuscada) + 3;
		// Trunc at 40 as maximum
		sprintf(buffer,"%s",salida);
		strtok(buffer,cadenaSplit);
		strcpy(ip,buffer);
	}
	Wifly_ExitCommandMode();

}

uint8_t Wifly_GetChannel(void)
{
	uint8_t lChan = 0;
	char* cadenaBuscada="Chan";

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); return -1; } // Error
	else
	{
		Wifly_SendCommand("get wlan",NULL,WIFLY_WAIT_SMALL_TIME, buffer_RX);
		char* salida = strstr(buffer_RX,cadenaBuscada) + 4;
		sprintf(salida,"%d",lChan);
		Wifly_ExitCommandMode();
		return lChan;
	}
}

int Wifly_GetRSSI(void)
{
	int lRSSI = 0;
	char* cadenaBuscada="RSSI";
	char* cadenaSplit=")";

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); return -1; } // Error
	else
	{
		Wifly_SendCommand("show rssi",NULL,WIFLY_WAIT_SMALL_TIME, buffer_RX);
		char* salida = strstr(buffer_RX,cadenaBuscada) + 6;
		strtok(salida,cadenaSplit);
		lRSSI = Utils_Jatoi(salida);
		Wifly_ExitCommandMode();
		return lRSSI;
	}
}

int Wifly_GetUptime(void)
{
	int lUPTIME = 0;
	char* cadenaBuscada="UpTime";
	char* cadenaSplit=" s";

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); return -1; } // Error
	else
	{
		Wifly_SendCommand("show time",NULL,WIFLY_WAIT_SMALL_TIME, buffer_RX);
		char* salida = strstr(buffer_RX,cadenaBuscada) + 7;
		strtok(salida,cadenaSplit);
		lUPTIME = Utils_Jatoi(salida);
		Wifly_ExitCommandMode();
		return lUPTIME;
	}
}

int Wifly_AmIConnected()
{
	int lclConnStatus=0;

	Wifly_ClearBuffers();
	UARTClear();

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); return -1; } // Error
	Wifly_SendCommand("show connection",NULL,WIFLY_WAIT_SMALL_TIME,buffer_RX);
	lclConnStatus=Utils_Jatoi(buffer_RX);
	if (wDEBUG) Log_Log("WiFLY INF: (show connection)\r\n%s\r\n",buffer_RX);
	Wifly_ExitCommandMode();
	return ((lclConnStatus & 0b0000000000010000) >> 4);
}

uint8_t Wifly_SendCommandAOK(const char *comando, const char *parametro)
{

		Wifly_ClearBuffers();
		UARTClear();

		Printf_Print(WiFly_SerialPort,"%s%s\r",comando,parametro);
		Wifly_ClearBuffers();
		Printf_Scan(WiFly_SerialPort,1000,"%s",buffer); /* Get returned command */
		if (wDEBUG) Log_Log("WiFLY RX: [*Command: %s %s] - [*Response: %s ]\r\n",comando, parametro, buffer);

		if (strcmp(buffer,"AOK")==0)
			return 1;
		else
			return 0;

}

uint8_t Wifly_SendCommand(const char *comando, const char *parametro, uint32_t demora, char *retorno)
{
	Wifly_ClearBuffers();
	UARTClear();

	Printf_Print(WiFly_SerialPort,"%s%s\r",comando,parametro);
	Printf_ScanCompleto(WiFly_SerialPort,demora,retorno); /* Get returned command */
	if (wDEBUG) Log_Log("WiFLY RX: %s\r\n",retorno);
	return 1;
}

int8_t Wifly_HTTPInit(uint8_t HTTPPort, char *dns_name)
{

	char lHTTPPort[4];
	if (http_Initialized == 1) return 1;

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); if(wDEBUG) Log_Log("[HTTP_Get] Error entering command mode"); return -1; } // Error
	if(Wifly_SendCommandAOK("set ip proto 18",0)!=1) { Wifly_ExitCommandMode(); return -1; }
	if(Wifly_SendCommandAOK("set option format 1",0)!=1)  { Wifly_ExitCommandMode(); return -1; }
	if(Wifly_SendCommandAOK("set dns name ",dns_name)!=1) { Wifly_ExitCommandMode(); return -1; }
	if(Wifly_SendCommandAOK("set ip host 0",0)!=1) { Wifly_ExitCommandMode(); return -1; }
	/* Convert to string */
	sprintf(lHTTPPort,"%i",HTTPPort);
	if(Wifly_SendCommandAOK("set ip remote ",lHTTPPort)!=1) { Wifly_ExitCommandMode(); return -1; }
	if(Wifly_SendCommandAOK("set com remote 0","\0")!=1) { Wifly_ExitCommandMode(); return -1; }

	Wifly_ExitCommandMode();
	http_Initialized = 1;
	return 1;

}

int8_t Wifly_HTTPGet(char *host,char *URI,char *httpget_buffer)
{

	if (http_Initialized == -1) Wifly_HTTPInit(80,host);

	if(Wifly_CommandMode() != 1) { Wifly_SetNoCommmandMode(); if(wDEBUG) Log_Log("[HTTP_Get] Error entering command mode"); return -1; } // Error
	if(Wifly_SendCommand("open ",host,1500,http_RX)!=1) { Wifly_ExitCommandMode(); return -1; }
		if (wDEBUG) Log_Log("OPEN Command: %s",http_RX);
	if(Wifly_SendCommand("GET ",URI,3000,http_RX)!=1) { Wifly_ExitCommandMode(); return -1; }
		if (wDEBUG) Log_Log("GET Command: %s",http_RX);

	//wifly_exitcommandMode();
		// The Get command exists me from command mode, so i manually set the module as if it were exited
		Wifly_SetNoCommmandMode();
	strcpy(httpget_buffer,http_RX);
	return 1;
}

void Wifly_ClearBuffers(void)
{
	memset(buffer_RX, 0, MAX_BUFFER);
	memset(buffer, 0, MAX_BUFFER);
}
